@@ -68,9 +68,9 @@ class BuilderTest(unittest.TestCase):
         spec4h1 spec4h2 spec5d1 spec5d2 spec5fi1 spec5fi2 spec5fi3 spec5fi4
         spec5f_ii_1 spec5f_ii_2 spec5f_ii_3 spec5f_ii_4
         spec5h1 spec6b_ii spec6d2 spec6e spec6f
-        spec6h_ii spec6h_iii_1 spec6h_iii_3d spec8a spec8b spec8c spec8d
-        spec9a spec9b spec9c1 spec9c2 spec9c3 spec9d spec9e spec9f spec9g
-        spec10
+        spec6h_ii spec6h_iii_1 spec6h_iii_3d spec8a spec8a_2 spec8b spec8c spec8d
+        spec9a spec9a2 spec9b spec9c1 spec9c2 spec9c3 spec9d spec9e spec9f
+        spec9g spec10
         bug453 bug457 bug463 bug501 bug502 bug504 bug505 bug506 bug509
         bug512 bug514 bug568 bug633 bug1307 bug1459 bug2276 variable_bug2772
         name size size2 multiple_feature_blocks omitted_GlyphClassDef
@@ -85,6 +85,15 @@ class BuilderTest(unittest.TestCase):
         variable_scalar_valuerecord variable_scalar_anchor variable_conditionset
         variable_mark_anchor duplicate_lookup_reference
         contextual_inline_multi_sub_format_2
+        contextual_inline_format_4
+        duplicate_language_stmt
+        CursivePosSubtable
+        MarkBasePosSubtable
+        MarkLigPosSubtable
+        MarkMarkPosSubtable
+        single_pos_NULL
+        class_pair_pos_duplicates
+        useExtension
     """.split()
 
     VARFONT_AXES = [
@@ -297,6 +306,71 @@ class BuilderTest(unittest.TestCase):
             r"Removing duplicate multiple substitution from glyph \"f_f_i\" to \('f', 'f', 'i'\)"
         )
 
+    def test_mixed_singleSubst_multipleSubst(self):
+        font = self.build(
+            "lookup test {"
+            "  sub f_f   by f f;"
+            "  sub f     by f;"
+            "  sub f_f_i by f f i;"
+            "  sub [A A.sc] by A;"
+            "  sub [B B.sc] by [B B.sc];"
+            "} test;"
+        )
+
+        assert "GSUB" in font
+        st = font["GSUB"].table.LookupList.Lookup[0].SubTable[0]
+        self.assertEqual(st.LookupType, 2)
+        self.assertEqual(
+            st.mapping,
+            {
+                "f_f": ("f", "f"),
+                "f": ("f",),
+                "f_f_i": ("f", "f", "i"),
+                "A": ("A",),
+                "A.sc": ("A",),
+                "B": ("B",),
+                "B.sc": ("B.sc",),
+            },
+        )
+
+    def test_mixed_singleSubst_ligatureSubst(self):
+        font = self.build(
+            "lookup test {"
+            "  sub f f   by f_f;"
+            "  sub f f i by f_f_i;"
+            "  sub A     by A.sc;"
+            "} test;"
+        )
+
+        assert "GSUB" in font
+        st = font["GSUB"].table.LookupList.Lookup[0].SubTable[0]
+        self.assertEqual(st.LookupType, 4)
+        self.assertEqual(len(st.ligatures), 2)
+        self.assertEqual(len(st.ligatures["f"]), 2)
+        self.assertEqual(st.ligatures["f"][0].LigGlyph, "f_f_i")
+        self.assertEqual(len(st.ligatures["f"][0].Component), 2)
+        self.assertEqual(st.ligatures["f"][0].Component[0], "f")
+        self.assertEqual(st.ligatures["f"][0].Component[1], "i")
+        self.assertEqual(st.ligatures["f"][1].LigGlyph, "f_f")
+        self.assertEqual(len(st.ligatures["f"][1].Component), 1)
+        self.assertEqual(st.ligatures["f"][1].Component[0], "f")
+        self.assertEqual(len(st.ligatures["A"]), 1)
+        self.assertEqual(st.ligatures["A"][0].LigGlyph, "A.sc")
+        self.assertEqual(len(st.ligatures["A"][0].Component), 0)
+
+    def test_mixed_singleSubst_multipleSubst_ligatureSubst(self):
+        self.assertRaisesRegex(
+            FeatureLibError,
+            "Within a named lookup block, all rules must be of the "
+            "same lookup type and flag",
+            self.build,
+            "lookup test {"
+            "  sub A     by A.sc;"
+            "  sub f_f   by f f;"
+            "  sub f f i by f_f_i;"
+            "} test;",
+        )
+
     def test_pairPos_redefinition_warning(self):
         # https://github.com/fonttools/fonttools/issues/1147
         logger = logging.getLogger("fontTools.otlLib.builder")
@@ -1177,6 +1251,14 @@ class BuilderTest(unittest.TestCase):
         addOpenTypeFeatures(font, feafile)
         assert dedent(str(feafile)) == dedent(features)
 
+    def test_feature_useExtension(self):
+        self.assertRaisesRegex(
+            FeatureLibError,
+            "'useExtension' keyword for feature blocks is allowed only for 'aalt' feature",
+            self.build,
+            "feature liga useExtension { sub f f by f_f; } liga;",
+        )
+
 
 def generate_feature_file_test(name):
     return lambda self: self.check_feature_file(name)
